// -------------------------------------------------------------------------------------
//
// roboFish.CPP                                                   Ulrich Mller 21.02.06
//                                                                PCS1700       VC++ 6.0
// Funktionen zum Betrieb des ROBO Interfaces auf Basis der TransferArea
//
// -------------------------------------------------------------------------------------

#include <windows.h>
#include <iostream.h>
#include "roboFish.H"

DWORD               res;
UINT                iStatusAlt;
USHORT              mModeStatus;
FT_HANDLE           usbHandle;
FT_TRANSFER_AREA*   ftDCB;
NOTIFICATION_EVENTS ftEvents;
int                 iCounter[8];
const UINT          rfMax = 150;
UINT                rfAnz;
UINT                rfIn;
UINT                rfOut;
SMESSAGE            rfMsg[rfMax];

void CCounter(void* TransferAreaAddress) {
	UINT iStatusNeu, iStatusDelta;
	UINT iMot, n, MotMode, MotDir;
	BOOL MotEnd;
	
	if(ftDCB == NULL) return;

	iStatusNeu   = ftDCB->E_Main;
	iStatusDelta = iStatusAlt ^ iStatusNeu;
	iStatusAlt   = iStatusNeu;

	for(iMot = 0; iMot < 4; iMot++) {
		n = iMot * 2;
		MotMode = (mModeStatus & (0x03 << n)) ? 1 : 0;
		switch(MotMode) {
		case 0: // --- NormalMode ---
				if((iStatusDelta & (0x01 << n))     > 0) iCounter[n]++;
				if((iStatusDelta & (0x01 << (n+1))) > 0) iCounter[n+1]++; 
				break;
		case 1: // --- RobMode ---
				if((iStatusDelta & (0x01 << (n+1))) > 0) iCounter[n+1]--;
				MotDir = (ftDCB->M_Main & (0x03 << n)) >> n;
				MotEnd = (ftDCB->E_Main & (0x01 << n)) ? TRUE : FALSE;
				if((iCounter[n+1] == 0) || ((MotDir == ftiLinks) && MotEnd)) {
					ftDCB->M_Main &= ~(0x03 << n);
					mModeStatus &= ~(0x03 << n);
					iCounter[n+1] = 0;
				}
				break;
		}
	}
}

void RFMessagesIn(SMESSAGE* inM) {
	if(rfAnz < rfMax) {
		rfMsg[rfIn].L.ucHwId  = inM->L.ucHwId;
		rfMsg[rfIn].L.ucSubId = inM->L.ucSubId;
		rfMsg[rfIn].L.dw      = inM->L.dw;
		rfIn = rfIn < rfMax ? rfIn + 1 : 0;
		rfAnz++;
	}
}

void InitFish(UINT DistanceMode) {
	cout      << "---InitFish gestartet ---" << endl;
	cout      << "Version : " << GetLibVersion() << endl;
	
	res       = InitFtLib();
	
	cout      << "Init : " << res << endl;
	
	res       = InitFtUsbDeviceList();
  usbHandle = GetFtUsbDeviceHandle(0);
	
	cout      << "DeviceTyp    " << GetFtDeviceTyp(usbHandle)        << endl;
	cout      << "SerialNr     " << GetFtSerialNrStrg(usbHandle)     << endl;
	cout      << "Firmware     " << GetFtFirmwareStrg(usbHandle)     << endl;
	cout      << "Manufacturer " << GetFtManufacturerStrg(usbHandle) << endl;
	cout      << "ShortName    " << GetFtShortNameStrg(usbHandle)    << endl;
	cout      << "LongName     " << GetFtLongNameStrg(usbHandle)     << endl;

	ftEvents.Context              = NULL;
	ftEvents.hEvent               = NULL;
	ftEvents.Message              = NULL;
	ftEvents.NotificationCallback = CCounter;
	ftEvents.CallbackMessage      = NULL;
	ftEvents.WindowsHandle        = NULL;

	res = OpenFtUsbDevice(usbHandle);
	res = SetFtDistanceSensorMode(usbHandle, DistanceMode, 0,0,0,0,0,0);
	if(GetFtDeviceTyp(usbHandle) == FT_ROBO_RF_DATA_LINK) {
		rfAnz = 0;
		rfIn  = 0;
		rfOut = 0;
		ftEvents.CallbackMessage = RFMessagesIn;
		StartFtTransferAreaWithCommunication(usbHandle, &ftEvents);
		Sleep(333);
	}
	else res = StartFtTransferArea(usbHandle, &ftEvents);
	ftDCB = GetFtTransferAreaAddress(usbHandle);
	for(int i = 0; i < 8; i++) iCounter[i] = 0;
	mModeStatus = 0;
	ftDCB->MPWM_Update = 0x01;
	cout      << "ENDE : Escape-Taste und RETURN" << endl;
}

void FinishFish(void) {
	
	char  Ende;

	res  = StopFtTransferArea(usbHandle);
	res  = CloseFtDevice(usbHandle);
	res  = CloseFtLib();
	cout << "--- FINIS ---" << endl;
  cin.get(Ende);
}

void ClearCounter(UCHAR InputNr) {
	iCounter[InputNr-1] = 0;
}

void ClearCounters() {
	UCHAR i;
	for(i = 0; i < 8; i++) iCounter[i] = 0;
}

void ClearRFMessagesIn() {
	rfAnz = 0;
	rfIn  = 0;
	rfOut = 0;
}

void ClearRFMessagesOut() {
	ClearFtMessageBuffer(usbHandle);
}

void ClearMotors() {
	ftDCB->M_Main = 0;
}

BOOL Finish(UCHAR InputNr) {
	if(GetAsyncKeyState(VK_ESCAPE) != 0) return TRUE;
	if(InputNr > 0) if(GetInput(InputNr)) return TRUE;
	return FALSE;
}

BOOL FinishIR(UCHAR IRCode, UCHAR IRKey) {
	if(GetIRKey(IRCode, IRKey)) return TRUE;
	return FALSE;
}

UINT GetAnalog(UCHAR AnalogNr) {
	switch(--AnalogNr) {
		case 0:  return ftDCB->AX; break;
		case 1:  return ftDCB->AY;
		default: return 0;
	}
}

UINT GetCounter(UCHAR InputNr) {
	return iCounter[InputNr-1];
}

UINT   GetDistanceValue(UCHAR DisNr) {
	switch (--DisNr) {
		case 0:  return ftDCB->D1; break;
		case 1:  return ftDCB->D2; break;
		default: return 0;
	}
}

BOOL GetInput(UCHAR InputNr) {
	return ftDCB->E16_Main[--InputNr] ? TRUE : FALSE;
}

UCHAR GetInputs() {
	return ftDCB->E_Main;
}

BOOL GetIRKey(UCHAR Code, UCHAR KeyNr) {
	KeyNr--;
	switch(Code) {
		case 0:
			return ftDCB->IRKeys16[KeyNr] ? TRUE : FALSE;
			break;
		case 1:
			return ftDCB->IRKeys16Code1[KeyNr] ? TRUE : FALSE;
			break;
		case 2:
			return ftDCB->IRKeys16Code2[KeyNr] ? TRUE : FALSE;
		default:
			return FALSE;
			break;
	}
}

UCHAR GetRFMessage(SMESSAGE* inM) {
	if(rfAnz > 0) {
		inM->L.ucHwId  = rfMsg[rfOut].L.ucHwId;
		inM->L.ucSubId = rfMsg[rfOut].L.ucSubId;
		inM->L.dw      = rfMsg[rfOut].L.dw;
		rfOut = rfOut < rfMax ? rfOut + 1 : 0;
		return 1;
	}
	else return 0;
}

UINT GetVoltage(UCHAR VoltNr) {
	switch(--VoltNr) {
		case 0:  return ftDCB->A1; break;
		case 1:  return ftDCB->A2; break;
		case 2:  return ftDCB->AV; break;
		case 3:  return ftDCB->AZ; break;
		case 4:  return ftDCB->D1; break;
		case 5:  return ftDCB->D2; break;
		default: return 0;
	}
}

BOOL IsRFMessage() {
	return rfAnz;
}

UCHAR Outputs(void) {
	return ftDCB->M_Main;
}

void Pause(UINT mSek) {
	Sleep(mSek);
}

UCHAR SendRFMessage(SMESSAGE* mData, UCHAR Spez) {
	res = SendFtMessage(usbHandle, MSG_HWID_RF, mData->L.ucSubId, mData->L.dw, 55, Spez);
	if(res) return 1; else return 0;
}

void SetCounter(UCHAR InputNr, UINT Value) {
	iCounter[InputNr-1] = Value;
}

void SetLamp(UCHAR LampNr, BOOL OnOff, UCHAR Power) {
	UINT n = --LampNr % 8;
	if(OnOff) ftDCB->M_Main |= 1 << n;
	else ftDCB->M_Main      &= ~(1 << n);
	ftDCB->MPWM_Main[n]      = --Power;
}

void  SetMotor(UCHAR MotorNr, UCHAR Dir, UCHAR Speed) {
	UCHAR n = --MotorNr << 1;
	ftDCB->M_Main &= ~(3 << n);
	switch(Dir) {
		case 0: break;
		case 1:
			ftDCB->M_Main |= ftiLinks << n;
			ftDCB->MPWM_Main[n]   = --Speed;
			break;
		case 2:
			ftDCB->M_Main |= ftiRechts << n;
			ftDCB->MPWM_Main[n+1] = --Speed;
			break;
	}
}

void SetRobMotor(UCHAR MotorNr, UCHAR Dir, UCHAR Speed, UINT Counter) {
	MotorNr--;
	UCHAR n = MotorNr * 2;
	mModeStatus &= ~(3 << n);
	mModeStatus |= 1 << n;
	iCounter[n+1] = Counter;
	ftDCB->M_Main &= ~(3 << n);
	if(Dir == ftiLinks) {
		ftDCB->M_Main |= ftiLinks << n;
		ftDCB->MPWM_Main[n] = --Speed;
	}
	else {
		ftDCB->M_Main |= ftiRechts << n;
		ftDCB->MPWM_Main[n+1] = --Speed;
	}

}

void  SetMotors(UCHAR MotorStatus, UINT SpeedStatus, UINT ModeStatus){
	UCHAR i, i2, i21;
	UCHAR Speed;
	ftDCB->M_Main = MotorStatus;
	for(i = 0; i < 4; i++) { 
		i2  = i * 2;
		i21 = i2 + 1;
		Speed = (SpeedStatus & (0x0F << (i*4))) >> (i*4);
		ftDCB->MPWM_Main[i2]  = Speed;
		ftDCB->MPWM_Main[i21] = Speed;
	}
	mModeStatus = ModeStatus;
}

void  WaitForChange(UCHAR InputNr, UINT NrOfChanges, UCHAR TermNr) {
	UINT INew, IOld;
	InputNr--;
	TermNr--;
	IOld = ftDCB->E16_Main[InputNr];
	do {
		if((TermNr > 0) & ftDCB->E16_Main[TermNr]) return;
		INew = ftDCB->E16_Main[InputNr];
		if(INew != IOld) {IOld = INew; NrOfChanges--;}
	} while(NrOfChanges);
}

void WaitForHigh(UCHAR InputNr) {
	WaitForInput(InputNr, FALSE);
	WaitForInput(InputNr, TRUE);
}

void WaitForInput(UCHAR InputNr, BOOL OnOff) {
	while(GetInput(InputNr) != OnOff);
}

void WaitForIR(UCHAR IRCode, UCHAR IRKey, BOOL OnOff) {
	do {
		Sleep(10);
	} while(!GetIRKey(IRCode, IRKey) == OnOff);
}

void WaitForLow(UCHAR InputNr) {
	WaitForInput(InputNr, TRUE);
	WaitForInput(InputNr, FALSE);
}

UCHAR  WaitForRFMessage(USHORT mSek, SMESSAGE* mData) {
	ftDCB->Timer1ms = mSek;
	do {
		if(GetRFMessage(mData))	return 1;
	} while(!mSek || ftDCB->Timer1ms);
	return 0;
}

UCHAR WaitForRobMotors(UINT mSek, UCHAR M1, UCHAR M2, UCHAR M3, UCHAR M4) {
	UINT c;
	ftDCB->Timer1ms = mSek;
	do {
		c = 0;
		if(M1) c += ftDCB->M_Main & (3 << ((M1-1)*2));
		if(M2) c += ftDCB->M_Main & (3 << ((M2-1)*2));
		if(M3) c += ftDCB->M_Main & (3 << ((M3-1)*2));
		if(M4) c += ftDCB->M_Main & (3 << ((M4-1)*2));
		if(!c) return 1;
	} while (!mSek || ftDCB->Timer1ms);
	return 0;
}

int WaitForPositionDown(UCHAR InputNr, int Counter, int Position, UCHAR TermNr) {
	WaitForChange(InputNr, Counter - Position, TermNr);
	return Counter - iCounter[--InputNr];
}

int WaitForPositionUp(UCHAR InputNr, int Counter, int Position, UCHAR TermNr) {
	WaitForChange(InputNr, Position - Counter, TermNr);
	return iCounter[--InputNr] + Counter;
}

void WaitForTime(USHORT mSek) {
	Pause(mSek);
}