// ----------------------------------------------------------------------------------
//
// roboFish.c                                                  Ulrich Mller 12.05.08
//                                                             PCS1700 Renesas C 4.00
// Betriebs-Routinen auf Basis der TransferArea
//
// ----------------------------------------------------------------------------------

#include "roboFish.h";

UCHAR iTimerTick;
UINT  iStatusAlt;
UINT  mModeStatus;
int   iCounter[8];
#define rfMax 150
UINT  rfAnz;
UINT  rfIn;
UINT  rfOut;
extern  struct s_Message sMsg;

void CCounter() {
  UINT iStatusNeu, iStatusDelta;
	UINT iMot, n, MotMode, MotDir;
	BOOL MotEnd;
  if(++iTimerTick < 10) return;
  iTimerTick = 0;
  iStatusNeu   = sTrans.E_Main;
  iStatusDelta = iStatusAlt ^ iStatusNeu;
  iStatusAlt   = iStatusNeu;
  for(iMot = 0; iMot < 4; iMot++) {
    n = iMot * 2;
    MotMode = (mModeStatus & (0x03 << n)) ? 1 : 0;
    switch(MotMode) {
    case 0:  // --- Normal Mode --------
      if((iStatusDelta & (0x01 << n))     > 0) iCounter[n]++;
      if((iStatusDelta & (0x01 << (n+1))) > 0) iCounter[n+1]++;
      break;
    case 1:
      if((iStatusDelta & (0x01 << (n+1))) > 0) iCounter[n+1]--;
      MotDir = (sTrans.M_Main & (0x03 << n)) >> n;
      MotEnd = (sTrans.E_Main & (0x01 << n)) ? TRUE : FALSE;
      if((iCounter[n+1] == 0) || ((MotDir == ftiLinks) && MotEnd)) {
        sTrans.M_Main &= ~(0x03 << n);  // --- Motor Ausschalten ---
        mModeStatus &= ~(0x03 << n);
        iCounter[n+1] = 0;
      }
      break;
    }
  }
}
  
void InitFish(BOOL rf, UINT DistanceMode) {                              
// --- Initialisierung ---
	sTrans.MPWM_Update = 0x01;        // --- Update PWM alle 10 ms
  iTimerTick = 0;SetFt1msTimerTickAddress((void far *)&CCounter);
  SetFtDistanceSensorMode(DistanceMode, 0,0,0,0,0,0);
  if(rf) {
    InitMessage();
    SetFtMessageReceiveAddress((void far*) &WriteMessageToBuffer);
  }
}

void FinishFish(void) {
  SetFt1msTimerTickAddress(NULL);
  SetFtMessageReceiveAddress(NULL);
}
void ClearCounter(UCHAR InputNr) {
	iCounter[InputNr-1] = 0;
}

void ClearCounters() {
	UCHAR i;
	for(i = 0; i < 8; i++) iCounter[i] = 0;
}

void  ClearMessagesIn() {
  ClearFtMessageBuffer();
}

void  ClearMessagesOut() {
}

void ClearMotors() {
	sTrans.M_Main = 0;
}

BOOL Finish(UCHAR InputNr) {
	if(InputNr > 0) if(GetInput(InputNr)) return TRUE;
	return FALSE;
}

BOOL FinishIR(UCHAR IRCode, UCHAR IRKey) {
	if(GetIRKey(IRCode, IRKey)) return TRUE;
	return FALSE;
}

UINT GetAnalog(UCHAR AnalogNr) {
  switch(--AnalogNr) {
	  case 0:  return sTrans.AX; break;
		case 1:  return sTrans.AY; break;
		default: return 0;
	}
}

UINT GetCounter(UCHAR InputNr) {
	return iCounter[InputNr-1];
}

UINT GetDistanceValue(UCHAR DisNr) {
  switch(--DisNr) {
    case 0: return sTrans.D1; break;
    case 1: return sTrans.D2; break;
    default: return 0;
  }
}
BOOL GetInput(UCHAR InputNr) {         // --- Abfrage I-Eingang
  return sTrans.E16_Main[--InputNr] ? TRUE : FALSE;
}

UCHAR GetInputs(void) {
  return sTrans.E_Main;
}

BOOL GetIRKey(UCHAR Code, UCHAR KeyNr) {
  KeyNr--;
	switch(Code) {
	  case 0:   // --- unabhngig vom Code
			return sTrans.IRKeys16[KeyNr] ? TRUE : FALSE;
			break;
		case 1: // --- Empfnger 1
			return sTrans.IRKeys16Code1[KeyNr] ? TRUE : FALSE;
			break;
		case 2:  // --- Empfnger 2
			return sTrans.IRKeys16Code2[KeyNr] ? TRUE : FALSE;
			break;
		default :
			return FALSE;
			break;
  }
}

UCHAR GetRFMessage(SMESSAGE* inM) {
  if(GetMessageFromBuffer(inM) == ERROR_SUCCESS) {
    return 1;
  }
  else {
    return 0;
  }
}

UINT GetVoltage(UCHAR VoltNr) {
  switch(--VoltNr) {
	  case 0:  return sTrans.A1; break;
		case 1:  return sTrans.A2; break;
    case 2:  return sTrans.AV; break;
    case 3:  return sTrans.AZ; break;
    case 4:  return sTrans.D1; break;
    case 5:  return sTrans.D2; break;
		default: return 0;
	}
}

BOOL  IsRFMessage() {
  return sMsg.ucCountMsg ? TRUE : FALSE;
}

UCHAR Outputs() {
	return sTrans.M_Main;
}

void Pause(UINT mSek) {
	FtDelay(mSek);
}

UCHAR SendRFMessage(SMESSAGE* mData, UCHAR Spez) {
  return SendFtMessage(MSG_HWID_RF, mData->L.ucSubId, mData->L.ulMsg, 55, Spez);
}

void SetLamp(UCHAR LampNr, BOOL OnOff, UCHAR Power) {
	UINT n = --LampNr % 8;
	if(OnOff) sTrans.M_Main |= 1 << n;
	else sTrans.M_Main &= ~(1 << n);
	sTrans.MPWM_Main[n] = --Power;
}

void SetMotor(UCHAR MotorNr, UCHAR Dir, UCHAR Speed) {  
// --- Schalten MotorAusgang mit var. Speed --------
 	UCHAR n = --MotorNr << 1;
  Speed--;
  sTrans.M_Main &= ~(3 << n);      // --- Richtung lschen
  switch (Dir) {                   //              setzen
    case 0: break;
	  case 1:                        // --- Links
			sTrans.M_Main |= 1 << n;
			sTrans.MPWM_Main[n] = Speed;  
			break; 
	  case 2:                        // --- Rechts
		  sTrans.M_Main |= 2 << n; 
			sTrans.MPWM_Main[n+1] = Speed;  
			break;
  }
}

void SetMotors(UCHAR MotorStatus, UINT SpeedStatus, UINT ModeStatus) {
  UCHAR i, i2, i21;
	UCHAR Speed;
	sTrans.M_Main = MotorStatus;
  for(i = 0; i < 4; i++) {
		i2  = i * 2;
		i21 = i2 + 1;
		Speed = (SpeedStatus & (0x0F << (i*4))) >> (i*4);
		sTrans.MPWM_Main[i2]  = Speed;
		sTrans.MPWM_Main[i21] = Speed;
	}
  mModeStatus = ModeStatus;
}

void SetRobMotor(UCHAR MotorNr, UCHAR Dir, UCHAR Speed, UINT Counter) {
	UCHAR n;
  n = --MotorNr * 2;
	mModeStatus   &= ~(3 << n);
	mModeStatus   |= 1 << n;
	iCounter[n+1]  = Counter;
	sTrans.M_Main &= ~(3 << n);
	if(Dir == ftiLinks) {
		sTrans.M_Main      |= ftiLinks << n;
		sTrans.MPWM_Main[n] = --Speed;
	}
	else {
		sTrans.M_Main        |= ftiRechts << n;
		sTrans.MPWM_Main[n+1] = --Speed;
	}

}

void WaitForChange(UCHAR InputNr, UINT NrOfChanges, UCHAR TermNr) { // --- Warten auf Anzahl Impulse
  UINT INew, IOld;
  InputNr--;
	TermNr--;
  IOld = sTrans.E16_Main[InputNr];                   // --- Merken Ausgangsstand des Eingangs
  do {
	  if(TermNr < 8 & sTrans.E16_Main[TermNr]) return;
    INew = sTrans.E16_Main[InputNr];                 // --- Feststellen aktueller Stand
	if(INew != IOld) { IOld = INew; NrOfChanges--;}    // --- Abfrage auf Vernderung
  } while(NrOfChanges);                              // --- Loopende, wenn NrOfChanges = 0
}
void WaitForInput(UCHAR InputNr, BOOL OnOff) {
	do {
    FtDelay(10);
  } while(GetInput(InputNr) != OnOff);
}

void WaitForIR(UCHAR IRCode, UCHAR IRKey, BOOL OnOff) {
	do {
		FtDelay(10);
	} while(!GetIRKey(IRCode, IRKey) == OnOff);
}

void WaitForHigh(UCHAR InputNr) {
  WaitForInput(InputNr, FALSE);
  WaitForInput(InputNr, TRUE);
}

void WaitForLow(UCHAR InputNr) {
  WaitForInput(InputNr, TRUE);
  WaitForInput(InputNr, FALSE);
}

UCHAR WaitForRFMessage(UINT mSek, SMESSAGE* mData) {
  sTrans.Timer1ms = mSek;
  do {
    if(GetRFMessage(mData)) return 1; 
  } while(!mSek || sTrans.Timer1ms);
  return 0;
}

UCHAR WaitForRobMotors(UINT mSek, UCHAR M1, UCHAR M2, UCHAR M3, UCHAR M4) {
	UINT c;
	sTrans.Timer1ms = mSek;
	do {
		c = 0;
		if(M1) c += sTrans.M_Main & (3 << ((M1-1)*2));
		if(M2) c += sTrans.M_Main & (3 << ((M2-1)*2));
		if(M3) c += sTrans.M_Main & (3 << ((M3-1)*2));
		if(M4) c += sTrans.M_Main & (3 << ((M4-1)*2));
		if(!c) return 1;
	} while (!mSek || sTrans.Timer1ms);
	return 0;
}

int WaitForPositionDown(UCHAR InputNr, int Counter, int Position, UCHAR TermNr) {
  WaitForChange(InputNr, Counter - Position, TermNr);
  return Counter - iCounter[--InputNr];
}

int WaitForPositionUp(UCHAR InputNr, int Counter, int Position, UCHAR TermNr) {
  WaitForChange(InputNr, Position - Counter, TermNr);
  return iCounter[--InputNr] + Counter;
}

void WaitForTime(UINT mSek) {
  Pause(mSek);
}